#include <iostream>
#include "getkey.h"
#include "matriz.h"
#include "pthread.h"
using namespace std;


void * MonitorKeys( void * a );

class Peca {
private:
    Matriz< char > peca;
    int lin;
    int col;
public:
    Peca() : peca( 4, 4 ), lin(0), col(0) {
    }
    Peca(int lin, int col) : peca( 4, 4 ), lin(lin), col(col) {
        for( int i = 0; i < 4; i++)
            for( int j = 0; j < 4; j++ )
                peca( i, j ) = ' ';    
    }
    
    Matriz<char> getPeca() {
        return peca;
    }
    
    void addBlock( int i, int j ) {
        peca(i,j) = '*';
    }

    void giraEsq() {
        Matriz< char > m( 4,4 );
        for( int i = 0; i < 4; i++)
            for( int j = 0; j < 4; j++ )
                m(i,j) = peca( 3 - j, i );    
        peca = m;
    }
    void giraDir() {
        Matriz< char > m( 4,4 ); 
        for( int i = 0; i < 4; i++)
            for( int j = 0; j < 4; j++ )
                m(i,j) = peca( j, 3 - i );    
        peca = m;
    }    
    
    int getLin() {
        return lin;
    }
    
    int getCol() {
        return col;
    }
    
    void setCol( int col ) {
        this->col = col;
    }    
    

    void setLin( int lin ) {
        this->lin = lin;
    }
    
    int getBorderMinCol() {
        int min = 5;
        for(int i = 0; i < 4; i++ ) {
            for( int j = 0; j < 4; j++ )
            if( peca(i,j) != ' ' && j < min )
                min = j;
        }
        return min;
    }
    
    int getBorderMaxCol() {
        int max = 0;
        for(int i = 0; i < 4; i++ ) {
            for( int j = 0; j < 4; j++ )
            if( peca(i,j) != ' ' && j > max )
                max = j;
        }
        return max;
    }    
};

class Area {
private:
    Matriz< char > area;
    Peca p;
public:
    Area(int nlinhas, int ncols) : area( nlinhas, ncols ), p(0,0)  {
        pthread_t t;
        pthread_create(&t, NULL, MonitorKeys, (void *)this);
        for(int i = 0; i < area.getNumLinhas(); i++ ) 
            for(int j = 0; j < area.getNumColunas(); j++ )
                area(i,j) = ' ';
                
        for(int i=0; i < ncols; i++ ) {
            area( 0, i ) = '#';
            area( nlinhas - 1, i ) = '#';
        }
        
        for(int i=1; i < nlinhas; i++ ) {
            area( i, 0 )  = '#';
            area( i, ncols - 1 ) = '#';
        }
    }
    
    void setPeca( Peca peca ) {
        this->p = peca;        
    }  
    
    Peca& getPeca() {
        return p;
    }
    bool hasColisionPeca() {
        for( int i = p.getLin(); i < p.getLin()+4; i++ )
            for( int j = p.getCol(); j < p.getCol()+4; j++ ) 
                if( p.getPeca()(i-p.getLin(),j-p.getCol()) != ' ' && area( i,j) != ' ')
                    return true;
        return false;                    
    }
    void giraPeca() {
        p.giraEsq();        
        
    } 
    void movePecaEsq() {
        int min = p.getBorderMinCol();
        if( p.getCol() > -min + 1 )
            p.setCol( p.getCol() -1 );        
    } 
    
    void movePecaDir() {
        int max = p.getBorderMaxCol();
        
        if( p.getCol() + max < area.getNumColunas() - 2)
            p.setCol( p.getCol() + 1 );        
    } 
    void blockPeca() {
        for( int i = p.getLin(); i < p.getLin()+4; i++ ) {
            for( int j = p.getCol(); j < p.getCol()+4; j++ ) {
                if( p.getPeca()(i-p.getLin(),j-p.getCol()) != ' ' )
                    area(i,j) = p.getPeca()(i-p.getLin(),j-p.getCol());
            }
        }
    }
    void downPeca() {
        while( needBlock() == false )
            p.setLin( p.getLin() + 1 );
        p.setLin( p.getLin() - 1 );
    }
    bool needBlock() {
        for(int j = 0; j < 4; j++ ) {
            for( int i = 3; i>=0; i--) {
                if( p.getPeca()(i, j) != ' ' ) {
                    if( area( i + p.getLin() + 1, j + p.getCol() ) != ' ' )
                        return true;     
                }
            }
        }
        return false;
    }
    int verificaPontuacao() {
        for(int i = 1; i < area.getNumLinhas() - 1; i++ ) 
            for(int j = 1; j < area.getNumColunas() - 1; j++ ) {
                if( area(i,j) == ' ' )
                    break;
                if( j == area.getNumColunas() - 2 )
                    return i;
            }
        return -1;
    }
    void reduceLine( int lin ) {
        for(int i = lin; i >= 2; i-- ) 
            for(int j = 1; j < area.getNumColunas() - 1; j++ )
                area(i,j ) = area( i - 1, j );
    }
    void print() {
        system("clear");
        Matriz <char > t(area.getNumLinhas(), area.getNumColunas() );
        for(int i = 0; i < area.getNumLinhas(); i++ )
            for(int j = 0; j < area.getNumColunas(); j++ )
                t(i,j) = area(i,j);
                        
        for( int i = p.getLin(); i < p.getLin()+4; i++ ) {
            for( int j = p.getCol(); j < p.getCol()+4; j++ ) {
                if( p.getPeca()(i-p.getLin(),j-p.getCol()) != ' ' )
                    t(i,j) = p.getPeca()(i-p.getLin(),j-p.getCol());
            }
        }
        for(int i = 0; i < area.getNumLinhas(); i++ ) {
            for(int j = 0; j < area.getNumColunas(); j++ ) {
                cout << t(i, j);
            }
            cout << endl;
        }
    }

};

void * MonitorKeys( void * a ) {
    int key;
    Area * area = (Area *) a;
    while( true ) { 
        key = GetKey();
        if( key == KEYRIGHT ) {
            area->movePecaDir();
            area->print();
        } else if( key == KEYLEFT ) {
            area->movePecaEsq();
            area->print();
        } else if( key == KEYUP ) {
            area->giraPeca();
            area->print();
        } else if( key == KEYDOWN ) {
            area->downPeca();
        }
    }
}

int main() {
    Area area(22, 22);
    Peca pecas[5];
    srand( time( NULL ) );

    // mini L
    pecas[0] = Peca( 0, 2);
    pecas[0].addBlock( 1,1 );
    pecas[0].addBlock( 2,1 );
    pecas[0].addBlock( 2,2 );
    
    // L
    pecas[1] = Peca( 0, 2);
    pecas[1].addBlock( 0,1 );
    pecas[1].addBlock( 1,1 );
    pecas[1].addBlock( 2,1 );
    pecas[1].addBlock( 2,2 );
    
    // Quadrado
    pecas[2] = Peca( 0, 2);
    pecas[2].addBlock( 1,2 );
    pecas[2].addBlock( 1,1 );
    pecas[2].addBlock( 2,1 );
    pecas[2].addBlock( 2,2 );
    
    // |
    pecas[3] = Peca( 0, 2);
    pecas[3].addBlock( 0,1 );
    pecas[3].addBlock( 1,1 );
    pecas[3].addBlock( 2,1 );
    pecas[3].addBlock( 3,1 );
    
    //  #
    //  ##
    //   #
    pecas[4] = Peca( 0, 2);
    pecas[4].addBlock( 0,1 );
    pecas[4].addBlock( 1,1 );
    pecas[4].addBlock( 1,2 );
    pecas[4].addBlock( 2,2 );
    
    area.setPeca( pecas[random() % 5] );
    int p;

    while( true ) {       
        area.print();
        area.getPeca().setLin( 1 + area.getPeca().getLin() );
        if( area.needBlock() ) {
            area.blockPeca();
            area.setPeca( pecas[random() % 5]  );
        }
        usleep( 100*1000 );
        while( (p = area.verificaPontuacao()) != -1 )
            area.reduceLine( p );
    }
}









